#!/usr/bin/perl
use strict;
use warnings;
use AutoDiff;
use Math::Matrix;

sub Jacobian {
  my $rf = shift;
  my $v = shift;
  my $dv = [ map {AutoDiff->new($_, 0)} @{$v}];
  my @j;
  foreach my $i (0..$#$v) {
    $dv->[$i]->set_der(1);
    push @j, [map {$_->[1]} @{$rf->($dv)}];
    $dv->[$i]->set_der(0);
  }
  Math::Matrix->new(@j)->transpose;
}

sub f {                              # function R^3 -> R^3
  my ($x, $y, $z) = @{$_[0]};
  return [ $x**2+$y**2+$z**2-1,
           2*$x**2+$y**2-4*$z,
           3*$x**2-4*$y+$z**2
         ];
}

my $v = [0.5, 0.5, 0.5];
print "x = $v->[0] y = $v->[1] z = $v->[2]\n";

for (1..6) {
  my $x = Math::Matrix->new($v)->transpose;
  my $fv = Math::Matrix->new(f($v))->transpose;
  my $j = Jacobian(\&f, $v);
  $x = $x - $j->invert*$fv;
  $v = [@{$x->transpose->[0]}];
  print "x = $v->[0] y = $v->[1] z = $v->[2]\n";
}
